/* SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only */
/* Copyright (c) 2023 Brett Sheffield <bacs@librecast.net> */

/* sync.h - Librecast data & file syncing API */

#ifndef _LC_SYNC_H
#define _LC_SYNC_H 1

#include <librecast/types.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>

#define NET_LOOPBACK 1

typedef uint8_t hash_t[HASHSIZE];
struct net_tree_s {
	uint64_t size;
	hash_t   tree[];
};

/* structure for passing stats from sync functions */
struct lc_stat_s {
	size_t bi_tot; /* bytes in: gross bytes transferred */
	size_t bi_inf; /* bytes in: information bytes */
};

ssize_t lc_recvtree(lc_ctx_t *lctx, unsigned char *hash, mtree_t *tree,
		lc_stat_t *stats, int flags);
ssize_t lc_recvchunk(lc_ctx_t *lctx, unsigned char *hash, void *data, const size_t len,
		lc_stat_t *stats, int flags);
ssize_t lc_sendtree(lc_ctx_t *lctx, unsigned char *hash, const mtree_t *tree,
		lc_stat_t *stats, int flags);
ssize_t lc_sendchunk(lc_ctx_t *lctx, unsigned char *hash, const void *data, const size_t len,
		lc_stat_t *stats, int flags);

/* local blob/file sync */
int lc_memsync(void *dst, void *src, const size_t n, q_t *q, lc_stat_t *stats, int flags);
int lc_syncfilelocal(const char *dst, const char *src, q_t *q, lc_stat_t *stats, int flags);

/* network sync */
ssize_t lc_sync(lc_ctx_t *lctx, unsigned char *hash, void *data, const size_t len,
		q_t *q, lc_stat_t *stats, int flags);
ssize_t lc_syncfile(lc_ctx_t *lctx, unsigned char *hash, const char *pathname,
		q_t *q, lc_stat_t *stats, int flags);

/* map local file into memory */
void *lc_mmapfile(const char *pathname, size_t *len, int prot, int flags, off_t offset,
		struct stat *sb);

#endif /* _LC_SYNC_H */
